/*
  Repeating WiFi Web Client

 This sketch connects to a a web server and makes a request
 using a WiFi equipped Arduino board.

 created 23 April 2012
 modified 31 May 2012
 by Tom Igoe
 modified 13 Jan 2014
 by Federico Vanzati

 http://www.arduino.cc/en/Tutorial/WifiWebClientRepeating
 This code is in the public domain.
 */

#include <SPI.h>
#include <WiFiNINA.h>
#include <ArduinoHttpClient.h>
#include <ArduinoJson.h>
#include "arduino_secrets.h"

#include <Wire.h>
#include <Adafruit_Sensor.h> 
#include <Adafruit_BME280.h>
#include "Adafruit_TSL2591.h"

#include <vector>

// Initialize sensors
Adafruit_BME280 bme; // BME sensor which has pressure, humidity and temperature
Adafruit_TSL2591 tsl = Adafruit_TSL2591(2591); // Luminosity sensor 

///////please enter your sensitive data in the Secret tab/arduino_secrets.h
char ssid[] = SECRET_SSID;        // your network SSID (name)
char pass[] = SECRET_PASS;    // your network password (use for WPA, or use as key for WEP)

int keyIndex = 0;            // your network key index number (needed only for WEP)

int status = WL_IDLE_STATUS;
boolean api_success = false;

// server address:
char server[] = "192.168.0.2";
IPAddress server_ip(192,168,0,2);
int port = 8080;

// Initialize the WiFi client library
WiFiClient client;
HttpClient http = HttpClient(client, server, port);

unsigned long lastConnectionTime = 0, lastMeasurement = 0;  // last time you connected to the server, in milliseconds
const unsigned long postingInterval = 60L * 10L * 1000L;     // delay (60 * 10 * 1000 = 10 minutes) between updates, in milliseconds
const unsigned long measurementInterval = 5L * 1000L;      // measure every 10 seconds

struct Data {
  int temp, humid, lumi;
};

struct Measurements {
  std::vector<Data> datas;

  void AddMeasurements(Data values) {
    datas.push_back(values);
  }

  Data AverageAndClear() {
    int temps = 0, humids = 0, lumis = 0;
    
    for(Data d : datas) {
      temps += d.temp;
      humids += d.humid;
      lumis += d.lumi;
    }
    
    int amount = datas.size();
    int av_temp = temps/amount;
    int av_humid = humids/amount;
    int av_lumi = lumis/amount;
    datas.clear();
    return Data{av_temp, av_humid, av_lumi};
  }
};

Measurements measurements;

void setup() {
  //Initialize serial and wait for port to open:
  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  // Initialize pins
  bme.begin(0x76);
  tsl.begin();
  tsl.setGain(TSL2591_GAIN_MED);
  tsl.setTiming(TSL2591_INTEGRATIONTIME_300MS);
  
  // check for the WiFi module:
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    // don't continue
    while (true);
  }

  // attempt to connect to WiFi network:
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);

    // wait 10 seconds for connection:
    delay(10000);
  }
  // you're connected now, so print out the status:
  printWifiStatus();
}

// MAIN LOOP
void loop() {
  // if there's incoming data from the net connection.
  // send it out the serial port.  This is for debugging
  // purposes only:
  while (client.available()) {
    char c = client.read();
    Serial.write(c);
  }

  // Take measurements and add them to measurements
  if(millis() - lastMeasurement > measurementInterval) {
    int temp = bme.readTemperature();
    int humidity = bme.readHumidity();
    int luminosity = tsl.getLuminosity(TSL2591_VISIBLE);
    measurements.AddMeasurements({temp, humidity, luminosity});
    lastMeasurement = millis();
  }
  if (millis() - lastConnectionTime > postingInterval) {
    httpRequest();
  }
}

void httpRequest() {
  // close any connection before send a new request.
  // This will free the socket on the NINA module
  Serial.println("making POST request");
  Data average = measurements.AverageAndClear();
  String contentType = "application/x-www-form-urlencoded";
  String postData ="temperature=";
         postData += average.temp;
         postData += "&humidity=";
         postData +=  average.humid;
         postData += "&luminosity=";
         postData += average.lumi;
         postData += "&parent_id=";
         postData += 1;
  
  http.post("/api/v1/nodeData", contentType, postData);

  // read the status code and body of the response
  int statusCode = http.responseStatusCode();
  String response = http.responseBody();

  Serial.print("Status code: ");
  Serial.println(statusCode);
  Serial.print("Response: ");
  Serial.println(response);
  
  // note the time that the connection was made:
  lastConnectionTime = millis();
}

// Tulostetaan wifin tiedot
void printWifiStatus() {
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // print your board's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
}
